@@ -1,27 +1,24 @@
-import logging
 import os
 import sys
 import time
-import errno
 import socket
+import logging
 from functools import partial
 from optparse import make_option
 from collections import defaultdict
-import subprocess
-
-from django.core.management.base import BaseCommand
-from django.db import connection
-import django
-from django.conf import settings
-from django.utils import timezone
-
-from multiprocessing import Process, Queue
+from multiprocessing import Queue
 
 import psutil
 
-from chroniker.models import Job, Log, order_by_dependencies
-from chroniker import constants as c
+from django import get_version, VERSION
+from django.core.management.base import BaseCommand
+from django.db import connection
+from django.utils import timezone
+
+from chroniker.models import Job, Log
 from chroniker import utils
+from chroniker import settings as _settings
+
 
 def kill_stalled_processes(dryrun=True):
     """
@@ -37,7 +34,7 @@ def kill_stalled_processes(dryrun=True):
         .values_list('current_pid', flat=True)))
     for pid in pids:
         try:
-            if utils.pid_exists(pid):# and not utils.get_cpu_usage(pid):
+            if utils.pid_exists(pid): # and not utils.get_cpu_usage(pid):
                 p = psutil.Process(pid)
                 cmd = ' '.join(p.cmdline())
                 if 'manage.py cron' in cmd:
@@ -45,28 +42,36 @@ def kill_stalled_processes(dryrun=True):
                     job = None
                     if jobs:
                         job = jobs[0]
-                    print('Killing process %s associated with %s.' % (pid, job))
+                    utils.smart_print('Killing process %s associated with %s.' % (pid, job))
                     if not dryrun:
                         utils.kill_process(pid)
                 else:
-                    'PID not cron.'
+                    print('PID not cron.')
             else:
                 print('PID dead.')
         except psutil.NoSuchProcess:
             print('PID does not exist.')
 
+
 class JobProcess(utils.TimedProcess):
-    
+
     def __init__(self, job, *args, **kwargs):
         super(JobProcess, self).__init__(*args, **kwargs)
         self.job = job
 
-def run_job(job, update_heartbeat=None, stdout_queue=None, stderr_queue=None, force_run=False, **kwargs):
-    
-    # TODO:causes UnicodeEncodeError: 'ascii' codec can't encode character u'\xa0' in position 59: ordinal not in range(128)
+
+def run_job(job, **kwargs):
+
+    update_heartbeat = kwargs.pop('update_heartbeat', None)
+    stdout_queue = kwargs.pop('stdout_queue', None)
+    stderr_queue = kwargs.pop('stderr_queue', None)
+    force_run = kwargs.pop('force_run', False)
+
+    # TODO:causes UnicodeEncodeError: 'ascii' codec can't encode
+    # character u'\xa0' in position 59: ordinal not in range(128)
     #print(u"Running Job: %i - '%s' with args: %s" \
     #    % (job.id, job, job.args))
-    
+
     # TODO:Fix? Remove multiprocess and just running all jobs serially?
     # Multiprocessing does not play well with Django's PostgreSQL
     # connection, as it seems Django's connection code is not thread-safe.
@@ -88,27 +93,36 @@ def run_job(job, update_heartbeat=None, stdout_queue=None, stderr_queue=None, fo
     #TODO:mark job as not running if still marked?
     #TODO:normalize job termination and cleanup outside of handle_run()?
 
-def run_cron(jobs=None, update_heartbeat=True, force_run=False, dryrun=False):
+
+def run_cron(jobs=None, **kwargs):
+
+    update_heartbeat = kwargs.pop('update_heartbeat', True)
+    force_run = kwargs.pop('force_run', False)
+    dryrun = kwargs.pop('dryrun', False)
+    clear_pid = kwargs.pop('clear_pid', False)
+    sync = kwargs.pop('sync', False)
+
     try:
-        
+
         # TODO: auto-kill inactive long-running cron processes whose
         # threads have stalled and not exited properly?
         # Check for 0 cpu usage.
         #ps -p <pid> -o %cpu
-        
+
         stdout_map = defaultdict(list) # {prod_id:[]}
         stderr_map = defaultdict(list) # {prod_id:[]}
         stdout_queue = Queue()
         stderr_queue = Queue()
-        
-        if settings.CHRONIKER_AUTO_END_STALE_JOBS and not dryrun:
+
+        if _settings.CHRONIKER_AUTO_END_STALE_JOBS and not dryrun:
             Job.objects.end_all_stale()
-            
+
         # Check PID file to prevent conflicts with prior executions.
         # TODO: is this still necessary? deprecate? As long as jobs run by
         # JobProcess don't wait for other jobs, multiple instances of cron
         # should be able to run simeltaneously without issue.
-        if settings.CHRONIKER_USE_PID:
+        if _settings.CHRONIKER_USE_PID:
+            pid_fn = _settings.CHRONIKER_PID_FN
             pid = str(os.getpid())
             any_running = Job.objects.all_running().count()
             if not any_running:
@@ -122,15 +136,14 @@ def run_cron(jobs=None, update_heartbeat=True, force_run=False, dryrun=False):
                         print('%s already exists, exiting' % pid_fn)
                         sys.exit()
                     else:
-                        print(('%s already exists, but contains stale '
-                            'PID, continuing') % pid_fn)
+                        print(('%s already exists, but contains stale ' 'PID, continuing') % pid_fn)
                 except ValueError:
                     pass
                 except TypeError:
                     pass
-            file(pid_fn, 'w').write(pid)
+            open(pid_fn, 'w').write(pid)
             clear_pid = True
-        
+
         procs = []
         if force_run:
             q = Job.objects.all()
@@ -138,10 +151,10 @@ def run_cron(jobs=None, update_heartbeat=True, force_run=False, dryrun=False):
                 q = q.filter(id__in=jobs)
         else:
             q = Job.objects.due_with_met_dependencies_ordered(jobs=jobs)
-        
+
         running_ids = set()
         for job in q:
-            
+
             # This is necessary, otherwise we get the exception
             # DatabaseError: SSL error: sslv3 alert bad record mac
             # even through we're not using SSL...
@@ -149,70 +162,74 @@ def run_cron(jobs=None, update_heartbeat=True, force_run=False, dryrun=False):
             # connections for each process by explicitly closing the
             # current connection.
             connection.close()
-            
+
             # Re-check dependencies to incorporate any previous iterations
             # that marked jobs as running, potentially causing dependencies
             # to become unmet.
             Job.objects.update()
             job = Job.objects.get(id=job.id)
             if not force_run and not job.is_due_with_dependencies_met(running_ids=running_ids):
-                print('Job %i %s is due but has unmet dependencies.' % (job.id, job))
+                utils.smart_print(u'Job {} {} is due but has unmet dependencies.'\
+                    .format(job.id, job))
                 continue
-            
+
             # Immediately mark the job as running so the next jobs can
             # update their dependency check.
-            print('Running job %i %s.' % (job.id, job))
+            utils.smart_print(u'Running job {} {}.'.format(job.id, job))
             running_ids.add(job.id)
             if dryrun:
                 continue
             job.is_running = True
             Job.objects.filter(id=job.id).update(is_running=job.is_running)
-            
+
             # Launch job.
-            #proc = JobProcess(job, update_heartbeat=update_heartbeat, name=str(job))
-            job_func = partial(
-                run_job,
-                job=job,
-                force_run=force_run or job.force_run,
-                update_heartbeat=update_heartbeat,
-                name=str(job),
-            )
-            proc = JobProcess(
-                job=job,
-                max_seconds=job.timeout_seconds,
-                target=job_func,
-                name=str(job),
-                kwargs=dict(
+            if sync:
+                # Run job synchronously.
+                run_job(
+                    job,
+                    update_heartbeat=update_heartbeat,
                     stdout_queue=stdout_queue,
                     stderr_queue=stderr_queue,
-                ))
-            proc.start()
-            procs.append(proc)
-        
+                    force_run=force_run or job.force_run,
+                )
+            else:
+                # Run job asynchronously.
+                job_func = partial(
+                    run_job,
+                    job=job,
+                    force_run=force_run or job.force_run,
+                    update_heartbeat=update_heartbeat,
+                    name=str(job),
+                )
+                proc = JobProcess(
+                    job=job,
+                    max_seconds=job.timeout_seconds,
+                    target=job_func,
+                    name=str(job),
+                    kwargs=dict(
+                        stdout_queue=stdout_queue,
+                        stderr_queue=stderr_queue,
+                    )
+                )
+                proc.start()
+                procs.append(proc)
+
         if not dryrun:
             print("%d Jobs are due." % len(procs))
-            
+
             # Wait for all job processes to complete.
             while procs:
-                
+
                 while not stdout_queue.empty():
                     proc_id, proc_stdout = stdout_queue.get()
                     stdout_map[proc_id].append(proc_stdout)
-                    
+
                 while not stderr_queue.empty():
                     proc_id, proc_stderr = stderr_queue.get()
                     stderr_map[proc_id].append(proc_stderr)
-                    
+
                 for proc in list(procs):
-                    
-                    # Auto kill processes that haven't terminated but yet
-                    # register no cpu usage.
-                    #cpu = proc.get_cpu_usage_recursive()
-                    #print('cpu:',proc,cpu)
-#                    if not cpu:
-#                        utils.kill_process(proc.pid)
-#                        time.sleep(1)
-                    
+
                     if not proc.is_alive():
                         print('Process %s ended.' % (proc,))
                         procs.remove(proc)
@@ -222,15 +239,16 @@ def run_cron(jobs=None, update_heartbeat=True, force_run=False, dryrun=False):
                         proc.terminate()
                         run_end_datetime = timezone.now()
                         procs.remove(proc)
-                        
+
                         connection.close()
                         Job.objects.update()
-                        run_start_datetime = Job.objects.get(id=proc.job.id).last_run_start_timestamp
+                        j = Job.objects.get(id=proc.job.id)
+                        run_start_datetime = j.last_run_start_timestamp
                         proc.job.is_running = False
                         proc.job.force_run = False
                         proc.job.force_stop = False
                         proc.job.save()
-                        
+
                         # Create log record since the job was killed before it had
                         # a chance to do so.
                         Log.objects.create(
@@ -241,20 +259,20 @@ def run_cron(jobs=None, update_heartbeat=True, force_run=False, dryrun=False):
                             on_time=False,
                             hostname=socket.gethostname(),
                             stdout=''.join(stdout_map[proc_id]),
-                            stderr=''.join(stderr_map[proc_id]+['Job exceeded timeout\n']),
+                            stderr=''.join(stderr_map[proc_id] + ['Job exceeded timeout\n']),
                         )
-                        
+
                 time.sleep(1)
-            print('!'*80)
+            print('!' * 80)
             print('All jobs complete!')
     finally:
-        if settings.CHRONIKER_USE_PID and os.path.isfile(pid_fn) \
-        and clear_pid:
+        if _settings.CHRONIKER_USE_PID and os.path.isfile(pid_fn) and clear_pid:
             os.unlink(pid_fn)
-            
+
+
 class Command(BaseCommand):
     help = 'Runs all jobs that are due.'
-    option_list = BaseCommand.option_list + (
+    option_list = getattr(BaseCommand, 'option_list', ()) + (
         make_option('--update_heartbeat',
             dest='update_heartbeat',
             default=1,
@@ -273,26 +291,59 @@ class Command(BaseCommand):
             dest='jobs',
             default='',
             help='A comma-delimited list of job ids to limit executions to.'),
+        make_option('--name',
+            dest='name',
+            default='',
+            help='A name to give this process.'),
+        make_option('--sync',
+            action='store_true',
+            default=False,
+            help='If given, runs jobs one at a time.'),
+        make_option('--verbose',
+            action='store_true',
+            default=False,
+            help='If given, shows debugging info.'),
     )
-    
+
+    def create_parser(self, prog_name, subcommand):
+        """
+        For ``Django>=1.10``
+        Create and return the ``ArgumentParser`` which extends ``BaseCommand`` parser with
+        chroniker extra args and will be used to parse the arguments to this command.
+        """
+        from distutils.version import StrictVersion # pylint: disable=E0611
+        parser = super(Command, self).create_parser(prog_name, subcommand)
+        version_threshold = StrictVersion('1.10')
+        current_version = StrictVersion(get_version(VERSION))
+        if current_version >= version_threshold:
+            parser.add_argument('--update_heartbeat',
+                dest='update_heartbeat',
+                default=1,
+                help='If given, launches a thread to asynchronously update ' + \
+                    'job heartbeat status.')
+            parser.add_argument('--force_run', dest='force_run', action='store_true', default=False, help='If given, forces all jobs to run.')
+            parser.add_argument('--dryrun', action='store_true', default=False, help='If given, only displays jobs to be run.')
+            parser.add_argument('--jobs', dest='jobs', default='', help='A comma-delimited list of job ids to limit executions to.')
+            parser.add_argument('--name', dest='name', default='', help='A name to give this process.')
+            parser.add_argument('--sync', action='store_true', default=False, help='If given, runs jobs one at a time.')
+            parser.add_argument('--verbose', action='store_true', default=False, help='If given, shows debugging info.')
+            self.add_arguments(parser)
+        return parser
+
     def handle(self, *args, **options):
-        pid_fn = settings.CHRONIKER_PID_FN
-        clear_pid = False
-        
+        verbose = options['verbose']
+        if verbose:
+            logging.basicConfig(level=logging.DEBUG)
+
         kill_stalled_processes(dryrun=False)
-        
+
         # Find specific job ids to run, if any.
-        jobs = [
-            int(_.strip())
-            for _ in options.get('jobs', '').strip().split(',')
-            if _.strip().isdigit()
-        ]
-        update_heartbeat = int(options['update_heartbeat'])
-        force_run = options['force_run']
+        jobs = [int(_.strip()) for _ in options.get('jobs', '').strip().split(',') if _.strip().isdigit()]
+
         run_cron(
             jobs,
-            update_heartbeat=update_heartbeat,
-            force_run=force_run,
+            update_heartbeat=int(options['update_heartbeat']),
+            force_run=options['force_run'],
             dryrun=options['dryrun'],
+            sync=options['sync'],
         )
-        
\ No newline at end of file
